package edu.jhmi.telometer.calc.impl;

import edu.jhmi.telometer.bean.Dimension;
import edu.jhmi.telometer.bean.Mask;
import edu.jhmi.telometer.calc.api.MaskLogicUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.List;

@Slf4j
public class MaskLogicUtilImpl implements MaskLogicUtil {

    @Override
    public Mask andMasks(Mask mask1, Mask mask2) {
        return combineMasks(mask1, mask2, true);
    }

    @Override
    public Mask orMasks(Mask mask1, Mask mask2) {
        return combineMasks(mask1, mask2, false);
    }

    private Mask combineMasks(Mask mask1, Mask mask2, boolean isAnd){
        Assert.isTrue(mask1.size() == mask2.size(), "Mask lengths aren't equal");
        List<Boolean> bools = new ArrayList();
        int length = mask1.size();
        for (int i = 0; i < length; i++){
            boolean newValue = isAnd ? (mask1.isPositionInMask(i) && mask2.isPositionInMask(i)) : (mask1.isPositionInMask(i) || mask2.isPositionInMask(i));
            bools.add(newValue);
        }
        return Mask.builder().dimension(mask1.getDimension()).bools(bools).build();
    }

    @Override
    public Mask getFullMask(Dimension dimension) {
        return getMaskSetToSameValue(dimension, true);
    }

    @Override
    public Mask getEmptyMask(Dimension dimension) {
        return getMaskSetToSameValue(dimension, false);
    }

    private Mask getMaskSetToSameValue(Dimension dimension, boolean b) {
        List<Boolean> bools = new ArrayList<>();
        int size = dimension.getWidth() * dimension.getHeight();
        for (int i = 0; i < size; i++){
            bools.add(b);
        }
        return Mask.builder().bools(bools).dimension(dimension).build();
    }
}
