package edu.jhmi.telometer.calc.impl;

import edu.jhmi.telometer.bean.Image;
import edu.jhmi.telometer.bean.Mask;
import edu.jhmi.telometer.bean.Rect;
import edu.jhmi.telometer.calc.api.ImagePlusConvertUtil;
import edu.jhmi.telometer.calc.api.MaskConvertUtil;
import edu.jhmi.telometer.calc.api.MaskLogicUtil;
import edu.jhmi.telometer.calc.api.MaskRoiUtil;
import edu.jhmi.telometer.calc.api.PositionConvertUtil;
import ij.ImagePlus;
import ij.gui.Roi;
import ij.process.ImageProcessor;
import org.springframework.util.Assert;

import java.awt.Rectangle;
import java.util.Set;

public class MaskRoiUtilImpl implements MaskRoiUtil {

    private ImagePlusConvertUtil imagePlusConvertUtil = new ImagePlusConvertUtilImpl();
    private PositionConvertUtil positionConvertUtil = new PositionConvertUtilImpl();
    private MaskConvertUtil maskConvertUtil = new MaskConvertUtilImpl();
    private MaskLogicUtil maskLogicUtil = new MaskLogicUtilImpl();

    @Override
    public Mask roiToMask(Roi roi) {
        Assert.notNull(roi, "roi is null");
        ImagePlus imagePlus = roi.getImage();
        Image image = imagePlusConvertUtil.imagePlusToImage(imagePlus);

        Rectangle rectangle = roi.getBounds();
        Assert.notNull(rectangle, "rectangle is null");
        Rect rect = positionConvertUtil.rectangleToRect(rectangle);
        Mask partialRectMask = maskConvertUtil.rectToRectPartialMask(rect);

        ImageProcessor maskImageProcessor = roi.getMask();
        if (maskImageProcessor != null){
            partialRectMask = maskConvertUtil.byteImageProcessorToMask(maskImageProcessor);
        }
        Mask mask = maskConvertUtil.rectPartialMaskToFullMask(partialRectMask, rect, image);
        return mask;
    }

    @Override
    public Mask roisToMask(Set<Roi> rois) {
        Assert.isTrue(!rois.isEmpty(), "rois is empty");
        Mask combinedMask = maskLogicUtil.getEmptyMask(imagePlusConvertUtil.imagePlusToDimension(rois.iterator().next().getImage()));
        for (Roi roi : rois){
            Mask roiMask = roiToMask(roi);
            combinedMask = maskLogicUtil.orMasks(roiMask, combinedMask);
        }
        return combinedMask;
    }
}
