package edu.jhmi.telometer.calc.impl;

import edu.jhmi.telometer.bean.Particle;
import edu.jhmi.telometer.bean.Point;
import edu.jhmi.telometer.calc.api.ParticleUtilHelper;
import lombok.extern.slf4j.Slf4j;

import java.util.HashSet;
import java.util.Set;

@Slf4j
public class ParticleUtilHelperImpl implements ParticleUtilHelper {


    private void populateParticle(Particle particle, Point point, Set<Point> availablePoints){
        Set<Point> unprocessedPoints = new HashSet();
        unprocessedPoints.add(point);
        while (!unprocessedPoints.isEmpty()){

            point = unprocessedPoints.iterator().next();

            particle.addPoint(point);
            unprocessedPoints.remove(point);
            availablePoints.remove(point);

            Set<Point> surroundingPointsTouching = surroundingPointsTouching(point, availablePoints);
            unprocessedPoints.addAll(surroundingPointsTouching);
        }
    }

    private static Set<Point> surroundingPointsTouching(Point point, Set<Point> availablePoints){
        Set<Point> surroundingPoints = surrondingPoints(point);
        surroundingPoints.retainAll(availablePoints);
        return surroundingPoints;
    }


    @Override
    public Set<Particle> pointsToParticles(Set<Point> points) {
        points = new HashSet(points); //defensive copy
        Set<Particle> particles = new HashSet();
        while (!points.isEmpty()){
            Point point = points.iterator().next();
            Particle particle = new Particle();

            populateParticle(particle, point, points);
            particles.add(particle);
        }
        return particles;
    }


    public static Set<Point> surrondingPoints(Point point) {
        Set<Point> points = new HashSet(8);
        for (int yOffset = -1; yOffset <= 1; yOffset++) { // up, center, then down
            for (int xOffset = -1; xOffset <= 1; xOffset++) { // left, center, then right
                if ((yOffset == 0) && (xOffset == 0)){
                    continue;
                }
                int x = point.getX() + xOffset;
                int y = point.getY() + yOffset;
                Point surroundingPoint = Point.builder().x(x).y(y).build();
                points.add(surroundingPoint);
            }
        }
        return points;
    }

}
