package edu.jhmi.telometer.calc.impl;

import edu.jhmi.telometer.bean.Pixelation;
import edu.jhmi.telometer.calc.api.StatUtilHelper;
import edu.jhmi.telometer.util.DoubleUtil;
import org.springframework.util.Assert;

import java.util.Comparator;

public class StatUtilHelperImpl implements StatUtilHelper {

    @Override
    public int max(Pixelation pixelation) {
        return pixelation.getPixels().stream().max(Comparator.comparing(Integer::valueOf)).get();
    }

    @Override
    public int min(Pixelation pixelation) {
        return pixelation.getPixels().stream().min(Comparator.comparing(Integer::valueOf)).get();
    }

    @Override
    public int area(Pixelation pixelation) {
        return pixelation.getPixels().size();
    }

    @Override
    public long sum(Pixelation pixelation) {
        long sum = 0;
        for (int s : pixelation.getPixels()){
            sum += s;
        }
        return sum;
    }

    @Override
    public long mean1000(Pixelation pixelation) {
        assertNotEmpty(pixelation);
        long sum = sum(pixelation);
        int area = area(pixelation);
        double mean = ((double) sum) / ((double) area);
        long mean1000 = DoubleUtil.doubleToLong1000(mean);
        return mean1000;
    }

    @Override
    public long stddev1000(Pixelation pixelation) {
        assertNotEmpty(pixelation);
        if (pixelation.getPixels().size() == 1){
            return 0;
        }
        double mean = DoubleUtil.long1000ToDouble(mean1000(pixelation));
        double sumOfDifferencesSquared = 0;
        for (int pixel  : pixelation.getPixels()){
            double difference = Math.abs(pixel - mean);
            double differencesSquared = (difference * difference);
            sumOfDifferencesSquared += differencesSquared;
        }
        int area = area(pixelation);
        double stdDev = Math.sqrt(sumOfDifferencesSquared / (area - 1));
        long stddev1000 = DoubleUtil.doubleToLong1000(stdDev);
        return stddev1000;
    }





    private static void assertNotEmpty(Pixelation pixelation){
        Assert.isTrue(!pixelation.getPixels().isEmpty(), "pixelation is empty");
    }




}
