package edu.jhmi.telometer.calc.impl;

import edu.jhmi.telometer.bean.Cell;
import edu.jhmi.telometer.bean.Telomere;
import edu.jhmi.telometer.bean.TelomereStat;
import edu.jhmi.telometer.calc.api.TelomereStatsCalculator;
import edu.jhmi.telometer.util.DoubleUtil;
import org.springframework.util.Assert;

import java.util.Collection;

public class TelomereStatsCalculatorImpl implements TelomereStatsCalculator {


    @Override
    public TelomereStat telomeresToStats(Collection<Telomere> telomeres, Cell cell) {
        telomeres.forEach(t -> Assert.isTrue(t.getCell().getId().equals(cell.getId()), "wrong telomere for cell: " + t + " cell: " + cell));
        TelomereStat telomereStat = new TelomereStat();
        if (telomeres.isEmpty()){
            return telomereStat;
        }
        telomereStat.setTelomereCount(telomeres.size());
        telomereStat.setTelomereSumSum(telomeres.stream().mapToLong(t -> t.getSum()).sum());
        telomereStat.setTelomereSumMax((int) telomeres.stream().mapToLong(t -> t.getSum()).max().getAsLong());
        telomereStat.setTelomereSumMin((int) telomeres.stream().mapToLong(t -> t.getSum()).min().getAsLong());
        telomereStat.setTelomereSumMean1000(DoubleUtil.doubleToLong1000(telomeres.stream().mapToLong(t -> t.getSum()).average().getAsDouble()));
        telomereStat.setTelomereAreaMean1000(DoubleUtil.doubleToLong1000(telomeres.stream().mapToLong(t -> t.getArea()).average().getAsDouble()));
        long cellSum = cell.getSum();
        double telometeSumSumToDapiSumPercent = 100.0 * ((double) telomereStat.getTelomereSumSum()) / ((double)  cellSum);
        telomereStat.setTelomereSumSumToCellSumPercent1000(DoubleUtil.doubleToLong1000(telometeSumSumToDapiSumPercent));
        return telomereStat;
    }

}
