package edu.jhmi.telometer.controller;

import edu.jhmi.telometer.interfce.HardConstants;
import edu.jhmi.telometer.service.api.AdminService;
import edu.jhmi.telometer.service.api.TableService;
import edu.jhmi.telometer.service.api.TreeService;
import edu.jhmi.telometer.util.DbPopulator;
import edu.jhmi.telometer.util.EnvironmentUtil;
import edu.jhmi.telometer.util.MainInit;
import edu.jhmi.telometer.util.MemoryUtil;
import edu.jhmi.telometer.util.api.ImageDirUtil;
import edu.jhmi.telometer.view.start.GeneralPanel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
@Slf4j
public class GeneralController {

    @Autowired
    private AdminService adminService;
    @Autowired
    private TableService tableService;
    @Autowired
    private TreeService treeService;
    @Autowired
    private ScoreController scoreController;
    @Autowired
    private ImageDirUtil imageDirUtil;
    @Autowired
    private DbPopulator dbPopulator;

    static {
        MainInit.initHeadless();
    }

    private final GeneralPanel generalPanel = new GeneralPanel();

    public void startup(){
        try{
            startupDelegate();
        } catch (Throwable e){
            log.error("Could not start GeneralController: " + e, e);
            throw e;
        }
    }

    private void startupDelegate() {
        log.info("Telometer Controller starting...");
        printInfo();
        if (!dbPopulator.areLookupTablesPopulated()){
            dbPopulator.populateDbLookupTables();
        }
        generalPanel.setImageDirUtil(imageDirUtil);
        generalPanel.setTreeService(treeService);
        generalPanel.setAdminService(adminService);
        generalPanel.setTableService(tableService);
        generalPanel.setScoreController(scoreController);
        generalPanel.display();
    }

    private void printInfo(){
        log.info("Telometer Version: {}", HardConstants.VERSION);
        log.info("Environment: {}", EnvironmentUtil.getEnvironmentStat());
        log.info("Memory: {}", MemoryUtil.getMemoryStat());
        log.info("Data Directory: {}", imageDirUtil.getDataDir().getAbsolutePath());
    }
}
