package edu.jhmi.telometer.controller;

import edu.jhmi.telometer.bean.Image;
import edu.jhmi.telometer.bean.LookupTables;
import edu.jhmi.telometer.bean.Mask;
import edu.jhmi.telometer.bean.Pixelation;
import edu.jhmi.telometer.bean.ImageMakeup;
import edu.jhmi.telometer.bean.ImageType;
import edu.jhmi.telometer.bean.Model;
import edu.jhmi.telometer.bean.NucleusEntry;
import edu.jhmi.telometer.bean.ScoreParams;
import edu.jhmi.telometer.bean.ScoreResults;
import edu.jhmi.telometer.bean.Step;
import edu.jhmi.telometer.bean.StepMode;
import edu.jhmi.telometer.bean.ToolMode;
import edu.jhmi.telometer.calc.api.GeneralUtil;
import edu.jhmi.telometer.interfce.ResultListener;
import edu.jhmi.telometer.interfce.ScorePanelListener;
import edu.jhmi.telometer.service.api.AdminService;
import edu.jhmi.telometer.service.api.TableService;
import edu.jhmi.telometer.service.api.TreeService;
import edu.jhmi.telometer.util.DialogUtil;
import edu.jhmi.telometer.util.DoubleUtil;
import edu.jhmi.telometer.util.MainInit;
import edu.jhmi.telometer.util.MemoryUtil;
import edu.jhmi.telometer.util.NucleusHolder;
import edu.jhmi.telometer.util.ToolModeUtil;
import edu.jhmi.telometer.thirdparty.Background_Subtracter_;
import edu.jhmi.telometer.thirdparty.Threshold_Adjuster_;
import edu.jhmi.telometer.bean.CardType;
import edu.jhmi.telometer.calc.api.ImagePlusConvertUtil;
import edu.jhmi.telometer.calc.api.MaskConvertUtil;
import edu.jhmi.telometer.calc.api.MaskLogicUtil;
import edu.jhmi.telometer.calc.api.MaskRoiUtil;
import edu.jhmi.telometer.calc.api.PixelModifier;
import edu.jhmi.telometer.calc.api.StatUtilHelper;
import edu.jhmi.telometer.calc.impl.ImagePlusConvertUtilImpl;
import edu.jhmi.telometer.calc.impl.MaskConvertUtilImpl;
import edu.jhmi.telometer.calc.impl.MaskLogicUtilImpl;
import edu.jhmi.telometer.calc.impl.MaskRoiUtilImpl;
import edu.jhmi.telometer.calc.impl.PixelModifierImpl;
import edu.jhmi.telometer.calc.impl.StatUtilHelperImpl;
import edu.jhmi.telometer.util.api.ImageDirUtil;
import edu.jhmi.telometer.view.admin.AdminPanel;
import edu.jhmi.telometer.view.query.table.TablesPanel;
import edu.jhmi.telometer.view.query.tree.TreePanel;
import edu.jhmi.telometer.view.score.ScorePanel;
import ij.ImagePlus;
import ij.WindowManager;
import ij.gui.ImageWindow;
import ij.gui.Roi;
import ij.gui.Toolbar;
import ij.io.Opener;
import ij.process.ImageConverter;
import ij.process.ImageProcessor;
import ij.process.ShortProcessor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import java.awt.Rectangle;
import java.io.File;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.prefs.Preferences;


@Component
public class ScoreController {

    private final StepMode STEP_MODE = StepMode.USER;

    private static final String LAST_USER_TELOMETER_INPUT_DIR = "LAST_USER_TELOMETER_INPUT_DIR";

    static {
        MainInit.initHeadless();
    }

    private static final Logger log = LoggerFactory.getLogger(ScoreController.class);

    private final ScorePanel scorePanel = new ScorePanel();
    private Step step;
    private final Model model = new Model();
    private final Preferences preferences = Preferences.userRoot().node(getClass().getName());
    private final JFileChooser imageFileChooser = new JFileChooser(preferences.get(LAST_USER_TELOMETER_INPUT_DIR,
            new File(".").getAbsolutePath()));
    private Threshold_Adjuster_ thresholdAdjustPluginDelegate = null;
    private final NucleusHolder nucleusHolder = model.getNucleusHolder();

    private ImagePlusConvertUtil imagePlusConvertUtil = new ImagePlusConvertUtilImpl();
    private MaskRoiUtil maskRoiUtil = new MaskRoiUtilImpl();
    private StatUtilHelper statUtilHelper = new StatUtilHelperImpl();
    private MaskConvertUtil maskConvertUtil = new MaskConvertUtilImpl();
    private PixelModifier pixelModifier = new PixelModifierImpl();
    private MaskLogicUtil maskLogicUtil = new MaskLogicUtilImpl();
    private TreePanel treePanel = new TreePanel();
    private TablesPanel tablesPanel = new TablesPanel();
    private AdminPanel adminPanel = new AdminPanel();

    @Autowired
    private GeneralUtil generalUtil;
    @Autowired
    private ImageDirUtil imageDirUtil;
    @Autowired
    private TreeService treeService;
    @Autowired
    private TableService tableService;
    @Autowired
    private AdminService adminService;

    private int cy3Background_normalization;
    private int dapiBackground_normalization;
    private ScoreResults scoreResults;
    private ResultListener resultListener;

    private ScorePanelListener scorePanelListener = new ScorePanelListener() {
        @Override
        public void nextButtonClicked() {
            handleNextButtonClicked();
        }

        @Override
        public void resetButtonClicked() {
            if (step != Step.COMPLETE) {
                String message = "Scoring isn't complete. Are you sure you want to reset without finishing and saving?";
                boolean wasYesClicked = DialogUtil.displayConfirmDialog(scorePanel.getParentComponent(),
                        message);
                if (!wasYesClicked) {
                    return;
                }
            }
            reset();
        }

        @Override
        public void minSliderAdjustment(int newValue) {
            if (thresholdAdjustPluginDelegate != null) {
                thresholdAdjustPluginDelegate.setMinValue(newValue);
            }
        }

        @Override
        public void addNucleusButtonClicked(NucleusEntry nucleusEntry) {
            String name = nucleusEntry.getName();
            if (name.trim().equals("")) {
                scorePanel.setInstructionsText("nucleus must have a name");
                return;
            }
            if (nucleusHolder.nameExists(name)) {
                scorePanel.setInstructionsText(String.format("name '%s' has already been used", name));
                return;
            }
            if (nucleusEntry.getCellType() == null) {
                scorePanel.setInstructionsText("no cellType.  Need to add in admin?");
                return;
            }
            if (nucleusEntry.getTissueType() == null) {
                scorePanel.setInstructionsText("no tissueType.  Need to add in admin?");
                return;
            }
            if (nucleusEntry.getLesionType() == null) {
                scorePanel.setInstructionsText("no lesionType.  Need to add in admin?");
                return;
            }

            ImageMakeup imageMakeup = model.getImageMakeup(ImageType.DAPI);
            ImagePlus imagePlus = imageMakeup.getImagePlus();
            Roi roi = imagePlus.getRoi();
            if (roi == null) {
                scorePanel.setInstructionsText("no ROI (region of interest) was drawn on the DAPI image");
                return;
            }

            nucleusEntry.setRoi(roi);
            imagePlus.killRoi();

            nucleusHolder.addNucleusEntry(nucleusEntry);
            scorePanel.getNucleiiPanel().addNucleusEntry(nucleusEntry);
            scorePanel.getNucleiiPanel().setName(nucleusHolder.getNextName());
            scorePanel.setNextButtonStatus(ScorePanel.NextButtonStatus.NEXT_ENABLED);
            String instructions = String.format("nucleus '%s' added", name);
            scorePanel.setInstructionsText(instructions);

            for (ImageType imageType : ImageType.values()) {
                ImagePlus ip = model.getImagePlus(imageType);
                if (ip != null) {
                    ip.updateAndRepaintWindow();
                }
            }
        }

        @Override
        public void removeNucleusButtonClicked(NucleusEntry nucleusEntry) {
            String name = nucleusEntry.getName();
            log.info("removing nucleus with name '{}'", name);

            ImageMakeup imageMakeup = model.getImageMakeup(ImageType.DAPI);
            nucleusHolder.removeByName(name);
            imageMakeup.getImagePlus().updateAndRepaintWindow();

            scorePanel.getNucleiiPanel().removeNucleusEntryByName(name);
            scorePanel.setNextButtonStatus(!nucleusHolder.isEmpty() ? ScorePanel.NextButtonStatus.NEXT_ENABLED
                    : ScorePanel.NextButtonStatus.NEXT_DISABLED);
            scorePanel.setInstructionsText(String.format("nucleus '%s' removed", name));
        }

        @Override
        public void fireDrawButtonClicked() {
            setToolModeOnCombined(ToolMode.DRAW);
        }

        @Override
        public void fireZoomButtonClicked() {
            setToolModeOnCombined(ToolMode.ZOOM);
        }

        @Override
        public void fireHandButtonClicked() {
            setToolModeOnCombined(ToolMode.HAND);
        }

        @Override
        public void viewResultsInTableClicked() {
            resultListener.scoringSelectedForTable(scoreResults.getScoring());
        }

        @Override
        public void viewResultsInTreeClicked() {
            resultListener.scoringSelectedForTree(scoreResults.getScoring());
        }
    };

    private void setToolModeOnCombined(ToolMode toolMode) {
        ImagePlus imagePlus = model.getImagePlus(ImageType.COMBINED);
        scorePanel.getToolbarPanel().setToolMode(toolMode);
        ToolModeUtil.setToolMode(toolMode, imagePlus);
    }


    public ScoreController() {
        scorePanel.setScorePanelListener(scorePanelListener);

    }

    private void handleNextButtonClicked() {
        try {
            handleNextButtonClickedDelegate();
        }
        catch (Throwable t) {
            String template = "Could not run next on Step '%s'.\nError: %s \n"
                    + "For help, email us a copy of telometer.log and your 2 images.\n";
            String message = String.format(template, step, t);
            log.error(message, t);
            log.info("Memory: {}", MemoryUtil.getMemoryStat());
            DialogUtil.displayErrorDialog(scorePanel.getParentComponent(), message);
        }
    }

    private void handleNextButtonClickedDelegate() {
        log.info("Starting Step {}", step);
        switch (step) {
            case START: {
                start();
                break;
            }
            case PRECHECK: {
                precheck();
                break;
            }
            case GATHER_INFO: {
                gatherInfo();
                break;
            }
            case PROCESS_INFO: {
                processInfo();
                break;
            }
            case SELECT_CY3_IMAGE: {
                selectImage(false);
                break;
            }
            case SELECT_DAPI_IMAGE: {
                selectImage(true);
                break;
            }
            case NORMALIZE_CY3_IMAGE: {
                normalizeImage(false);
                break;
            }
            case PROCESS_NORMALIZE_CY3_IMAGE: {
                processNormalizeImage(false);
                break;
            }
            case NORMALIZE_DAPI_IMAGE: {
                normalizeImage(true);
                break;
            }
            case PROCESS_NORMALIZE_DAPI_IMAGE: {
                processNormalizeImage(true);
                break;
            }
            case ROLLING_BALL_BACKGROUND_SUBTRACT: {
                rollingBallBackgroundSubtract();
                break;
            }
            case PROCESS_ROLLING_BALL_BACKGROUND_SUBTRACT: {
                processRollingBallBackgroundSubtract();
                break;
            }
            case THRESHOLDING: {
                thresholding();
                break;
            }
            case PROCESS_THRESHOLDING: {
                processThresholding();
                break;
            }
            case ADD_NUCLEII: {
                addNucleii();
                break;
            }
            case REMOVE_HALOS: {
                removeHalos();
                break;
            }
            case COMPLETE: {
                complete();
                break;
            }
            default: {
                throw new RuntimeException("bad step: " + step);
            }
        }
    }


    private File getImageFile(boolean isDapi) {
        if (STEP_MODE.isAuto()) {
            String dapiFileName = "TEST_100X_dapi.tif"; //  "60X_colon_01_dapi.tif"; //
            String cy3FileName = "TEST_100X_cy3.tif"; //60X_colon_01_cy3.tif"; //TEST_100X_cy3.tif";
            return isDapi ?
                    new File(String.format("/Users/jim/dev/telometer/telometer-app/src/main/scripts/sample_images/%s", dapiFileName)) :
                    new File(String.format("/Users/jim/dev/telometer/telometer-app/src/main/scripts/sample_images/%s", cy3FileName));
        } else {
            String description = getCy3orDapiName(isDapi);
            String title = String.format("Select the %s Image", description);
            imageFileChooser.setDialogTitle(title);
            int response = imageFileChooser.showOpenDialog(scorePanel.getParentComponent());
            if (response != JFileChooser.APPROVE_OPTION) {
                return null;
            }
            File file = imageFileChooser.getSelectedFile();
            return file;
        }
    }

    private void gatherInfo() {
        String instructions = "Set options for this scoring";
        scorePanel.setInstructionsText(instructions);
        scorePanel.setResetButtonEnabled(true);
        showImageExclusive();
        scorePanel.getOptionsPanel().setTag(generalUtil.findDefaultImageTag());
        scorePanel.setCardPanelView(CardType.GATHER_INFO);

        step = Step.PROCESS_INFO;
        if (STEP_MODE.isAuto()) {
            handleNextButtonClicked();
        }
    }

    private void processInfo() {
        if (scorePanel.getOptionsPanel().getPreset() == null) {
            scorePanel.setInstructionsText("no preset was selected");
            return;
        }
        if (scorePanel.getOptionsPanel().getProject() == null) {
            scorePanel.setInstructionsText("no project was selected");
            return;
        }
        String tag = scorePanel.getOptionsPanel().getTag();
        if (tag == null || tag.trim().equals("")) {
            scorePanel.setInstructionsText("no tag was selected");
            return;
        }
        if (generalUtil.isTagAlreadyUsed(tag)) {
            scorePanel.setInstructionsText("tag has already been used: " + tag);
            return;
        }
        step = Step.SELECT_CY3_IMAGE;
        handleNextButtonClicked();
    }


    private void selectImage(boolean isDapi) {
        String description = getCy3orDapiName(isDapi);
        String instructions = String.format("Select the %s Image\n%s", description, getCy3orDapiFullDescription(isDapi));
        scorePanel.setInstructionsText(instructions);
        scorePanel.setCardPanelView(CardType.NONE);
        showImageExclusive();

        File file = getImageFile(isDapi);
        if (file == null) {
            scorePanel.setInstructionsText(String.format("No file was selected. Click next to try again to select a %s file", description));
            return;
        }
        if (!file.exists()) {
            scorePanel.setInstructionsText(String.format("file '%s' does not exist. Click next to try again to select a %s file", file.getAbsolutePath(), description));
            return;
        }
        preferences.put(LAST_USER_TELOMETER_INPUT_DIR, file.getParent());
        log.info("Opening ImagePlus: {}", file.getAbsolutePath());
        Opener opener = new Opener();
        ImagePlus imagePlus = opener.openImage(file.getAbsolutePath());
        String reasonRejected = imagePlusConvertUtil.getReasonImagePlusNotAcceptable(imagePlus);
        if (reasonRejected != null) {
            scorePanel.setInstructionsText(String.format("selected file for %s not okay: %s", description, reasonRejected));
            return;
        }
        ImageType imageType = isDapi ? ImageType.DAPI : ImageType.CY3;
        model.initModelImagePlus(imagePlus, imageType);
        scorePanel.setInstructionsText(String.format("%s image selected", description));
        step = isDapi ? Step.NORMALIZE_CY3_IMAGE : Step.SELECT_DAPI_IMAGE;
        if (STEP_MODE.notDebug()) {
            handleNextButtonClicked();
        }
    }


    private void normalizeImage(boolean isDapi) {
        String description = getCy3orDapiName(isDapi);
        String instructions = String.format("Normalize the %s image: "
                        + "This substracts the average intensity of an area of background.\n"
                        + "Highlight an area of the background (black) that is devoid of features and\n then click the 'Next' button to normalize the image\n",
                description);
        scorePanel.setInstructionsText(instructions);
        Toolbar.getInstance().setTool(Toolbar.RECTANGLE);
        showImageExclusive(isDapi ? ImageType.DAPI : ImageType.CY3);
        step = isDapi ? Step.PROCESS_NORMALIZE_DAPI_IMAGE : Step.PROCESS_NORMALIZE_CY3_IMAGE;

        ImagePlus imagePlus = isDapi ? model.getDapiImagePlus() : model.getCy3ImagePlus();
        imagePlus.setRoi(new Rectangle(0, 0, 50, 50));

        if (STEP_MODE.isAuto()) {
            handleNextButtonClicked();
        }
    }

    private void showImageExclusive(ImageType... imageTypes) {
        Set<ImageType> imageTypeSet = new HashSet(Arrays.asList(imageTypes));
        for (ImageType i : ImageType.values()) {
            boolean isImageTypeVisible = imageTypeSet.contains(i);
            setImageTypeVisible(i, isImageTypeVisible);
        }
    }

    private void setImageTypeVisible(ImageType imageType, boolean isVisible) {
        ImageMakeup imageMakeup = model.getImageMakeup(imageType);
        if (imageMakeup == null) {
            return;
        }
        ImageWindow imageWindow = imageMakeup.getImageWindow();
        ImagePlus imagePlus = imageMakeup.getImagePlus();
        if (isVisible) {
            imageWindow.setVisible(true);
            WindowManager.setCurrentWindow(imagePlus.getWindow());
        } else {
            imageWindow.setVisible(false);
            imagePlus.killRoi();
        }

    }

    private void processNormalizeImage(boolean isDapi) {
        String description = getCy3orDapiName(isDapi);
        ImagePlus imagePlus = isDapi ? model.getDapiImagePlus() : model.getCy3ImagePlus();

        Roi roi = imagePlus.getRoi();

        if (roi == null) {
            scorePanel.setInstructionsText(String.format("Can't normalize %s image: There was no rectangle drawn on the image\n" +
                    "Draw a rectangle on the image background (black) and click the 'next' button", description));
            return;
        }
        Mask mask = maskRoiUtil.roiToMask(roi);
        Image image = imagePlusConvertUtil.imagePlusToImage(imagePlus);
        Pixelation squarePixelation = maskConvertUtil.imageAndMaskToPixelation(image, mask);
        int mean = (int) Math.round(DoubleUtil.long1000ToDouble(statUtilHelper.mean1000(squarePixelation)));
        if (isDapi) {
            dapiBackground_normalization = mean;
        } else {
            cy3Background_normalization = mean;
        }
        Pixelation pixelation = image.getPixelation();
        pixelation = pixelModifier.subtractIntensity(pixelation, mean);
        ShortProcessor processor = (ShortProcessor) imagePlus.getProcessor();
        processor.setPixels(imagePlusConvertUtil.pixelationToShortArray(pixelation));
        processor.resetMinAndMax();
        imagePlus.updateAndDraw();
        scorePanel.setInstructionsText(String.format("%s image normalized", description));
        step = isDapi ? Step.ROLLING_BALL_BACKGROUND_SUBTRACT : Step.NORMALIZE_DAPI_IMAGE;
        if (STEP_MODE.isAuto()) {
            handleNextButtonClicked();
        }
    }

    private void rollingBallBackgroundSubtract() {
        String instructions = "Create a 'masked cy3 image' by running the rolling ball background subtractor.\n"
                + "The rolling ball radius is automatically selected from the preset mode you have chosen.\n"
                + "Click the next button to continue";
        showImageExclusive(ImageType.CY3);
        scorePanel.setInstructionsText(instructions);
        step = Step.PROCESS_ROLLING_BALL_BACKGROUND_SUBTRACT;
        if (STEP_MODE.isAuto()) {
            handleNextButtonClicked();
        }
    }

    private void processRollingBallBackgroundSubtract() {
        ImagePlus imagePlus = model.getCy3ImagePlus();
        Image image = imagePlusConvertUtil.imagePlusToImage(imagePlus);
        Pixelation pixelation = image.getPixelation();
        pixelation = Pixelation.copy(pixelation);

        ImageProcessor imageProcessor = imagePlus.getProcessor().createProcessor(image.getDimension().getWidth(), image.getDimension().getHeight());
        imageProcessor.setPixels(imagePlusConvertUtil.pixelationToShortArray(pixelation));
        imagePlus = new ImagePlus("Masked CY - ", imageProcessor);
        model.initModelImagePlus(imagePlus, ImageType.MASKED_CY3);

        ImageConverter converter = new ImageConverter(imagePlus);
        converter.convertToGray8();
        imageProcessor = imagePlus.getProcessor();

        imageProcessor.resetMinAndMax();
        imagePlus.updateAndDraw();

        Background_Subtracter_ background_subtracter = new Background_Subtracter_();
        background_subtracter.subtractBackround(imageProcessor, scorePanel.getOptionsPanel().getPreset().getRollingBallSize());

        showImageExclusive(ImageType.CY3, ImageType.MASKED_CY3);
        scorePanel.setInstructionsText("'Masked Cy3 image' created by running rolling ball background subtraction on cy3 image.\n"
                + "Click 'next button' to continue");
        step = Step.THRESHOLDING;
        if (STEP_MODE.isAuto()) {
            handleNextButtonClicked();
        }
    }

    private void thresholding() {
        String instructions = "Threshold the 'masked cy3 image'.\n"
                + "Adjust the slider to find a level that maximizes the size of the spots but does not introduce too much noise.\n" +
                "A small amount of noise is okay as this will be automatically removed.\n";
        scorePanel.setInstructionsText(instructions);
        showImageExclusive(ImageType.MASKED_CY3);
        thresholdAdjustPluginDelegate = new Threshold_Adjuster_(); //calls third party plugin
        scorePanel.setCardPanelView(CardType.THRESHOLDER);
        scorePanel.getThresholderPanel().setMinSliderValue(thresholdAdjustPluginDelegate.getMinValue());
        step = Step.PROCESS_THRESHOLDING;
        if (STEP_MODE.isAuto()) {
            handleNextButtonClicked();
        }
    }

    private void addNucleii() {
        String instructions = "Select Nuclei for Analysis\n"
                + "Draw a freehand ROI around a nucleus of interest in the DAPI image and click the 'Add Nucleus' button.\n"
                + "Repeat this process until all of the desired nuclei have been selected. Press \"Next\" when finished.\n";

        scorePanel.setInstructionsText(instructions);
        scorePanel.setCardPanelView(CardType.ADD_NUCLEII);
        scorePanel.getNucleiiPanel().setName(nucleusHolder.getNextName());
        showImageExclusive(ImageType.DAPI, ImageType.CY3);
        Toolbar.getInstance().setTool(Toolbar.FREEROI);
        scorePanel.setNextButtonStatus(ScorePanel.NextButtonStatus.NEXT_DISABLED);
        step = Step.REMOVE_HALOS;
        if (STEP_MODE.isAuto()) {
            ImagePlus imagePlus = model.getImagePlus(ImageType.DAPI);
            Roi roi = new Roi(140, 210, 70, 70);
            roi.setImage(imagePlus);
            imagePlus.setRoi(roi);
            scorePanel.getNucleiiPanel().fireAddNucleusButtonClicked();
            //handleNextButtonClicked();
        }
    }


    private void complete() {
        long start = System.currentTimeMillis();
        String instructions = "Calculations complete.  Data stored to your computer.\n"
                + "Click button below to view data or click 'Reset' to score another image pair\n"
                + "WE DON'T PROVIDE BACKUPS OR STORE YOUR DATA REMOTELY--\n"
                + "This data is stored only on your machine. (Click 'About' button for details)";
        scorePanel.setInstructionsText(instructions);
        scorePanel.setCardPanelView(CardType.COMPLETE);
        scorePanel.setNextButtonStatus(ScorePanel.NextButtonStatus.HIDDEN);
        showImageExclusive(ImageType.COMBINED);

        ImagePlus annotatedImagePlus = imageDirUtil.getAnnotatedImagePlus(model);
        model.initModelImagePlus(annotatedImagePlus, ImageType.ANNOTATED);

        ScoreParams scoreParams = ScoreParams.builder()
                .project(scorePanel.getOptionsPanel().getProject())
                .tag(scorePanel.getOptionsPanel().getTag())
                .cy3Background(cy3Background_normalization)
                .dapiBackground(dapiBackground_normalization)
                .preset(scorePanel.getOptionsPanel().getPreset())
                .isCy3Subtracted(scorePanel.getOptionsPanel().getSubtractCy3())
                .build();
        scoreResults = generalUtil.calcScoreResults(scoreParams, model);
        generalUtil.dumpScoreResultsToDatabase(scoreResults);

        imageDirUtil.dumpImages(scoreResults.getScoring(), model);

        treePanel.getTreeOnlyPanel().setTreeService(treeService);
        tablesPanel.setTableService(tableService);
        treePanel.getTreeOnlyPanel().setScoring(scoreResults.getScoring());
        tablesPanel.setScoring(scoreResults.getScoring());
        adminPanel.setAdminService(adminService);
        long end = System.currentTimeMillis();
        log.info("Complete took {} millis", (end - start));
    }


    private void removeHalos() {
        String instructions = "Remove halos and seperate conjoined particles (Optional)\n"
                + "Zoom in on one of the chosen nuclei until the FISH objects are easily recognized.\n"
                + "Often, some objects (telomeres) will have merged together, while others may have halos.";
        scorePanel.setInstructionsText(instructions);
        scorePanel.setCardPanelView(CardType.REMOVE_HALOS);

        createCombinedImage();
        showImageExclusive(ImageType.COMBINED);
        setToolModeOnCombined(ToolMode.HAND);
        step = Step.COMPLETE;
        scorePanel.setNextButtonStatus(ScorePanel.NextButtonStatus.COMPLETE_ENABLED);
        if (STEP_MODE.isAuto()) {
            //handleNextButtonClicked();
        }
    }

    /**
     * Part 1: Get the mask: Get the intersection of the masked CY3 image (done by thresholder)
     * and the masked Dapi image (drawn by user)
     * Part 2: Using that mask, grab the pixels from the original Cy3 image (after it was normalized)
     */
    private void createCombinedImage() {
        Mask mask = getCombinedCircledNucleiAndMaskedCy3Mask();
        ImagePlus cy3ImagePlus = model.getImagePlus(ImageType.CY3);
        Image image = imagePlusConvertUtil.imagePlusToImage(cy3ImagePlus);
        image = pixelModifier.zeroOutNonMask(image, mask);

        ShortProcessor imageProcessor = new ShortProcessor(cy3ImagePlus.getWidth(), cy3ImagePlus.getHeight());
        imageProcessor.setPixels(imagePlusConvertUtil.pixelationToShortArray(image.getPixelation()));
        ImagePlus imagePlus = new ImagePlus("combined image", imageProcessor);
        model.initModelImagePlus(imagePlus, ImageType.COMBINED);
    }

    private Set<Roi> nucleusHolderRois() {
        Set<Roi> rois = new HashSet<>();
        for (NucleusEntry nucleusEntry : nucleusHolder.getNucleusEntrys()) {
            rois.add(nucleusEntry.getRoi());
        }
        return rois;
    }

    private Mask getCombinedCircledNucleiAndMaskedCy3Mask() {
        Mask circledNucleiiMask = getCircledNucleiiMask();
        Mask maskedCy3Mask = getMaskedCy3Mask();
        Mask mask = maskLogicUtil.andMasks(circledNucleiiMask, maskedCy3Mask);
//        ImagePlus imagePlus = maskConvertUtil.maskToImagePlus(mask);
//        imagePlus.setTitle("combined mask");
//        imagePlus.show();
        return mask;
    }

    private Mask getCircledNucleiiMask() {
        Set<Roi> rois = nucleusHolderRois();
        Mask mask = maskRoiUtil.roisToMask(rois);
//        ImagePlus imagePlus = maskConvertUtil.maskToImagePlus(mask);
//        imagePlus.setTitle("roi's mask");
//        imagePlus.show();
        return mask;
    }


    private Mask getMaskedCy3Mask() {
        ImagePlus imagePlus = model.getImagePlus(ImageType.MASKED_CY3);
//        byte[] byteMask = (byte[]) imagePlus.getProcessor().getPixels();
//        boolean[] mask = roiHelperUtil.byteMaskToMask(byteMask);
        Mask mask = maskConvertUtil.byteImageProcessorToMask(imagePlus.getProcessor());

//        ImagePlus imagePlus1 = maskConvertUtil.maskToImagePlus(mask);
//        imagePlus1.setTitle("maskedCy3 mask");
//        imagePlus1.show();
        return mask;
    }


    private void processThresholding() {
        double despeckleRatio = DoubleUtil.long1000ToDouble(scorePanel.getOptionsPanel().getPreset().getDespeckleRatio1000());
        thresholdAdjustPluginDelegate.apply(despeckleRatio);
        scorePanel.setInstructionsText("thresholding applied. Click next button to proceed to adding nuclei");
        step = Step.ADD_NUCLEII;
        if (STEP_MODE.notDebug()) {
            handleNextButtonClicked();
        }
    }


    private void start() {
        scorePanel.setResetButtonEnabled(false);
        scorePanel.setNextButtonStatus(ScorePanel.NextButtonStatus.NEXT_ENABLED);
        scorePanel.setInstructionsText("To begin, click the next button to start the scoring process");
        step = Step.PRECHECK;
        if (STEP_MODE.isAuto()) {
            handleNextButtonClicked();
        }
    }

    private void precheck() {
        log.info("performing precheck...");
        LookupTables lookupTables = generalUtil.lookupTables();
        String error = "No %s found. Click the 'Admin' button to add one";
        if (lookupTables.getPresets().isEmpty()) {
            scorePanel.setInstructionsText(String.format(error, "Preset"));
            return;
        }
        if (lookupTables.getProjects().isEmpty()) {
            scorePanel.setInstructionsText(String.format(error, "Project"));
            return;
        }
        if (lookupTables.getCellTypes().isEmpty()) {
            scorePanel.setInstructionsText(String.format(error, "CellType"));
            return;
        }
        if (lookupTables.getLesionTypes().isEmpty()) {
            scorePanel.setInstructionsText(String.format(error, "LesionType"));
            return;
        }
        if (lookupTables.getTissueTypes().isEmpty()) {
            scorePanel.setInstructionsText(String.format(error, "TissueType"));
            return;
        }
        init();
        step = Step.GATHER_INFO;
        handleNextButtonClicked();
    }


    private void reset() {
        log.info("Resetting ScoreController...");
        nucleusHolder.clear();
        scorePanel.getNucleiiPanel().clear();
        clearModel();

        step = Step.START;
        scorePanel.setCardPanelView(CardType.NONE);
        scorePanel.setScorePanelVisible(true);
        handleNextButtonClicked();
    }

    private void clearModel() {
        for (ImageType imageType : ImageType.values()) {
            closeImagePlus(imageType);
        }
    }

    private void closeImagePlus(ImageType imageType) {
        ImageMakeup imageMakeup = model.getImageMakeup(imageType);
        if (imageMakeup == null) {
            return;
        }
        model.clearImageMakeup(imageType);
        ImagePlus imagePlus = imageMakeup.getImagePlus();
        //imagePlus.set
        if (imagePlus != null) {
            //imagePlus.hide();
            imagePlus.killRoi();
            imagePlus.killStack();
        }
        if (imagePlus == null) {
            return;
        }
        ImageWindow imageWindow = imageMakeup.getImageWindow();
        if (imageWindow == null) {
            return;
        }
        imageWindow.close();
    }

    private void init() {
        log.info("performing init");
        MainInit.initImageJ();
        LookupTables lookupTables = generalUtil.lookupTables();
        scorePanel.getOptionsPanel().setPresets(lookupTables.getPresets());
        scorePanel.getOptionsPanel().setProjects(lookupTables.getProjects());
        scorePanel.getNucleiiPanel().setTissueTypes(lookupTables.getTissueTypes());
        scorePanel.getNucleiiPanel().setCellTypes(lookupTables.getCellTypes());
        scorePanel.getNucleiiPanel().setLesionTypes(lookupTables.getLesionTypes());
        treePanel.getNodeInfoPanel().setImageDirUtil(imageDirUtil);
    }

    public void begin() {
        if (step == null) { //first time
            reset();
        } else {
            log.info("Displaying ScorePanel");
            scorePanel.setScorePanelVisible(true); //just display the frame
        }
    }

    private static String getCy3orDapiName(boolean isDapi) {
        String description = isDapi ? "Dapi" : "Cy3";
        return description;
    }

    private static String getCy3orDapiFullDescription(boolean isDapi) {
        String description = getCy3orDapiName(isDapi);
        final String fileDescription = !isDapi ? "FISH signals" : "images of the cell nuclei (we use the DNA-binding dye DAPI for this purpose)";
        String full = String.format("The %s is the %s in your matched pair of TIFF images.\n(the one with the %s)",
                description, !isDapi ? "first" : "second", fileDescription);
        return full;
    }


    public void setResultListener(ResultListener resultListener) {
        this.resultListener = resultListener;
    }
}

