package edu.jhmi.telometer.service.impl;

import edu.jhmi.telometer.bean.Cell;
import edu.jhmi.telometer.bean.Project;
import edu.jhmi.telometer.bean.Scoring;
import edu.jhmi.telometer.bean.Telomere;
import edu.jhmi.telometer.interfce.Ided;
import edu.jhmi.telometer.interfce.Named;
import edu.jhmi.telometer.repo.CellRepo;
import edu.jhmi.telometer.repo.ProjectRepo;
import edu.jhmi.telometer.repo.ScoringRepo;
import edu.jhmi.telometer.repo.TelomereRepo;
import edu.jhmi.telometer.service.api.TableService;
import edu.jhmi.telometer.service.api.TreeService;
import edu.jhmi.telometer.util.CellTransientPopulator;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Component
@Slf4j
public class TableServiceImpl implements TableService {

    @Autowired
    private ProjectRepo projectRepo;

    @Autowired
    private ScoringRepo scoringRepo;

    @Autowired
    private CellRepo cellRepo;

    @Autowired
    private TelomereRepo telomereRepo;

    private CellTransientPopulator cellTransientPopulator = new CellTransientPopulator();

    @Override
    public List<Project> findAllProjects() {
        return sort(projectRepo.findAll());
    }

    @Override
    public List<Scoring> findAllScorings() {
        return sort(scoringRepo.findAll());
    }

    @Override
    public List<Scoring> findScoringsByProjects(Set<Project> projects) {
        List<Scoring> scorings =  scoringRepo.findScoringsByProjectIn(projects);
        return sort(scorings);
    }

    @Override
    public List<Cell> findCellsInScorings(Set<Scoring> scorings) {
        List<Cell> cells = sort(cellRepo.findCellsByScoringIn(scorings));
        List<Telomere> telomeres = telomereRepo.findTelomeresByCellIn(new HashSet(cells));
        cellTransientPopulator.populateCellsTransientFields(cells, telomeres);
        return cells;
    }

    @Override
    public List<Telomere> findTelomeresInCells(Set<Cell> cells) {
        List<Telomere> telomeres = telomereRepo.findTelomeresByCellIn(cells);
        return sort(telomeres);
    }

    private static<T extends Ided> List<T> sort(List<T> ideds){
        Collections.sort(ideds, new NamedComparator());
        return ideds;

    }

    private static class NamedComparator implements Comparator<Ided>{
        @Override
        public int compare(Ided ided1, Ided ided2) {
            return ided1.getId().compareTo(ided2.getId());
        }
    }
}


