package edu.jhmi.telometer.thirdparty;

import ij.IJ;
import ij.ImagePlus;
import ij.ImageStack;
import ij.Undo;
import ij.WindowManager;
import ij.gui.Toolbar;
import ij.measure.Measurements;
import ij.plugin.filter.PlugInFilter;
import ij.plugin.frame.Recorder;
import ij.process.ByteProcessor;
import ij.process.ImageProcessor;
import ij.process.ImageStatistics;
import ij.process.StackProcessor;

/** This plugin implements the Proxess/Binary/Threshold command. */
public class Thresholder_
        implements PlugInFilter, Measurements {
    public static int lastMinThreshold = 0;
    public static int lastMaxThreshold = 0;

    //private int slice;
    private double minThreshold;
    private double maxThreshold;
    boolean autoThreshold;
    ImageStack stack1;
    static boolean fill1 = false;
    static boolean fill2 = false;
    static boolean useBW = true;

    public int setup(String arg, ImagePlus imp) {
        if (arg.equals("about")) {
            return DONE;
        }
        return DOES_16 + DOES_8G;
    }

    public void run(ImageProcessor ip) {
        ImagePlus imp = WindowManager.getCurrentImage();
        if (imp == null) {
            IJ.noImage();
            return;
        }
        if (imp.getType() == ImagePlus.COLOR_RGB) {
            IJ.error("RGB images are not supported.");
            return;
        }
        if (imp.getStackSize() == 1)
            Undo.setup(Undo.COMPOUND_FILTER, imp);
        else
            Undo.reset();
        applyThreshold(imp);
    }

    void applyThreshold(ImagePlus imp) {
        imp.killRoi();
        ImageProcessor ip = imp.getProcessor();
        double saveMinThreshold = ip.getMinThreshold();
        double saveMaxThreshold = ip.getMaxThreshold();
        double saveMin = ip.getMin();
        double saveMax = ip.getMax();
        lastMinThreshold = (int) saveMinThreshold;
        lastMaxThreshold = (int) saveMaxThreshold;

        autoThreshold = saveMinThreshold == ImageProcessor.NO_THRESHOLD;
        if (!(imp.getType() == ImagePlus.GRAY8))
            convertToByte(imp);
        ip = imp.getProcessor();
        if (autoThreshold)
            autoThreshold(imp);
        else {
            if (Recorder.record)
                Recorder.record("setThreshold", (int) saveMinThreshold, (int) saveMaxThreshold);
            minThreshold = ((saveMinThreshold - saveMin) / (saveMax - saveMin)) * 255.0;
            maxThreshold = ((saveMaxThreshold - saveMin) / (saveMax - saveMin)) * 255.0;
        }

        lastMinThreshold = (int) minThreshold;
        lastMaxThreshold = (int) maxThreshold;

        boolean useBlackAndWhite = true;
        if (!autoThreshold) {
            /*
             * GenericDialog gd = new GenericDialog("Apply Lut");
             * gd.addCheckbox("Thresholded pixels to foreground color", fill1);
             * gd.addCheckbox("Remaining pixels to background color", fill2);
             * gd.addMessage(""); gd.addCheckbox("White foreground, black background",
             * useBW); gd.showDialog(); if (gd.wasCanceled()) {return;} fill1 =
             * gd.getNextBoolean(); fill2 = gd.getNextBoolean(); useBW = useBlackAndWhite =
             * gd.getNextBoolean();
             */
            fill1 = false;
            fill2 = false;
            useBW = useBlackAndWhite = true;
        } else {
            fill1 = true;
            fill2 = true;
        }

        int fcolor = 0, bcolor = 0;
        ip.resetThreshold();
        //int savePixel = ip.getPixel(0, 0);
        if (useBlackAndWhite) {
            // ip.setColor(Color.white);
            fcolor = 0xFF;
            fill1 = true;
            fill2 = true;
        } else
            ip.setColor(Toolbar.getForegroundColor());
        // ip.drawPixel(0,0);
        // fcolor = ip.getPixel(0,0);
        if (useBlackAndWhite)
            // ip.setColor(Color.black);
            bcolor = 0x00;
        else
            ip.setColor(Toolbar.getBackgroundColor());
        // ip.drawPixel(0,0);
        // bcolor = ip.getPixel(0,0);
        // ip.setColor(Toolbar.getForegroundColor());
        // ip.putPixel(0,0,savePixel);

        int[] lut = new int[256];
        for (int i = 0; i < 256; i++) {
            if (i >= minThreshold && i <= maxThreshold)
                lut[i] = fill1 ? fcolor : (byte) i;
            else {
                lut[i] = fill2 ? bcolor : (byte) i;
            }
        }
        int result = IJ.setupDialog(imp, 0);
        if (result == PlugInFilter.DONE) {
            return;
        }
        if (result == PlugInFilter.DOES_STACKS)
            new StackProcessor(imp.getStack(), ip).applyTable(lut);
        else
            ip.applyTable(lut);
        imp.updateAndDraw();

    }

    void convertToByte(ImagePlus imp) {
        // ImageProcessor ip = imp.getProcessor();
        // imp.setProcessor(null, ip.convertToByte(true));
        ImageProcessor ip = imp.getProcessor();
        double min = ip.getMin();
        double max = ip.getMax();
        int currentSlice = imp.getCurrentSlice();
        stack1 = imp.getStack();
        ImageStack stack2 = imp.createEmptyStack();
        int nSlices = imp.getStackSize();
        String label;
        for (int i = 1; i <= nSlices; i++) {
            label = stack1.getSliceLabel(1);
            ip = stack1.getProcessor(i);
            ip.setMinAndMax(min, max);
            stack2.addSlice(label, ip.convertToByte(true));
        }
        imp.setStack(null, stack2);
        imp.setSlice(currentSlice);
        imp.getCalibration().disableDensityCalibration();
    }

    void autoThreshold(ImagePlus imp) {
        ImageStatistics stats = imp.getStatistics(MIN_MAX + MODE);
        ImageProcessor ip = imp.getProcessor();
        int threshold = ((ByteProcessor) ip).getAutoThreshold();
        if ((stats.max - stats.mode) < (stats.mode - stats.min)) {
            minThreshold = stats.min;
            maxThreshold = threshold;
        } else {
            minThreshold = threshold;
            maxThreshold = stats.max;
        }
    }

}
