package edu.jhmi.telometer.util;

import ch.qos.logback.core.util.LocationUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.StringUtils;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Optional;
import java.util.concurrent.atomic.AtomicBoolean;

@Slf4j
public class DialogUtil {

    public static void displayErrorDialog(Component parentComponent, String message){
        JOptionPane.showMessageDialog(parentComponent, message, "Error",
                JOptionPane.ERROR_MESSAGE);
    }

    //Method created because sometimes JOptionPane.showMessageDialog would appear with only part
    //of the title bar showing.
    public static void displayInfoDialog(Component parentComponent, String message){
        log.info("Displaying dialog: {}", StringUtil.trim(message, 100));
        boolean isLongMessage = message.length() > 999;
        JTextArea textField = new JTextArea(message);
        textField.setEditable(false);

        JScrollPane scrollPane = new JScrollPane(textField);
        JButton closeButton = new JButton("Close");
        JPanel panel1 = new JPanel(new BorderLayout());
        if (!isLongMessage) {
            panel1.add(new JLabel(IconUtil.getIcon("info")), BorderLayout.WEST);
        }
        panel1.add(scrollPane, BorderLayout.CENTER);
        panel1.add(closeButton, BorderLayout.SOUTH);
        textField.setBackground(Color.LIGHT_GRAY);
        panel1.setBackground(Color.LIGHT_GRAY);

        Frame frame = JOptionPane.getFrameForComponent(parentComponent);
        JDialog dialog = new JDialog(frame, "Message", true);
        closeButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                dialog.dispose();
            }
        });
        dialog.getContentPane().add(panel1);
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        final int widthOfScreen = (int) screenSize.getWidth();
        final int heightOfScreen = (int) screenSize.getHeight();
        int width = widthOfScreen / 3;
        int height = !isLongMessage ? (int) (heightOfScreen / 4) : (int) (heightOfScreen / 2) ;
        dialog.setPreferredSize(new Dimension(width, height));
        dialog.pack();
        FrameUtil.setRelativeLocation(dialog, parentComponent);
        dialog.setVisible(true);
    }


    public static boolean displayConfirmDialog(Component parentComponent, String message){
        final JLabel icon = new JLabel(IconUtil.getIcon("warning"));
        final AtomicBoolean wasYesClicked = new AtomicBoolean(false);

        JTextArea textField = new JTextArea(message);
        textField.setBackground(Color.LIGHT_GRAY);
        textField.setEditable(false);
        JScrollPane scrollPane = new JScrollPane(textField);
        JPanel bottomPanel = new JPanel();
        JButton yesButton = new JButton("Yes");
        JButton cancelButton = new JButton("Cancel");
        bottomPanel.add(yesButton);
        bottomPanel.add(cancelButton);
        JPanel mainPanel = new JPanel(new BorderLayout());
        mainPanel.add(bottomPanel, BorderLayout.SOUTH);
        mainPanel.add(scrollPane, BorderLayout.CENTER);
        mainPanel.add(icon, BorderLayout.WEST);
        Frame frame = JOptionPane.getFrameForComponent(parentComponent);
        JDialog dialog = new JDialog(frame, "Message", true);

        cancelButton.addActionListener(a -> dialog.dispose());
        yesButton.addActionListener(a -> {
            wasYesClicked.set(true);
            dialog.dispose();
        });

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        final int widthOfScreen = (int) screenSize.getWidth();
        final int heightOfScreen = (int) screenSize.getHeight();
        int width = widthOfScreen / 3;
        int height = heightOfScreen / 6;
        dialog.setPreferredSize(new Dimension(width, height));

        dialog.getContentPane().add(mainPanel);
        dialog.pack();
        FrameUtil.setRelativeLocation(dialog, parentComponent);

        dialog.setVisible(true);
        return wasYesClicked.get();
    }

}
