package edu.jhmi.telometer.util;

import lombok.Data;

import java.text.NumberFormat;

public class MemoryUtil {
    
    private static final NumberFormat percentMemoryFreeNumberFormat = NumberFormat.getInstance();
    private static final NumberFormat memoryNumberFormat = NumberFormat.getInstance();
    private static final float MEGABYTE = 1024f * 1024f;

    @Data
    public static class MemoryStat{
        public String usedMemoryString;
        public String totalMemoryString;
        public String maxMemoryString;
        public String percentMemoryUsedString;
    }

    public static MemoryStat getMemoryStat(){
        Runtime runtime = Runtime.getRuntime();
        long freeMemory = runtime.freeMemory(); // the amount of free memory in the jvm
        long maxMemory = runtime.maxMemory(); // the highest amount of memory the jvm has allocated
        long totalMemory = runtime.totalMemory(); // current amount of memory the jvm has allocated
        long usedMemory = totalMemory - freeMemory;
        float percentMemoryUsed = 100.0f * ((float) usedMemory) / ((float) maxMemory);

        float usedMemoryInMegs = ((float) usedMemory) / MEGABYTE;
        float totalMemoryInMegs = ((float) totalMemory) / MEGABYTE;
        float maxMemoryInMegs = ((float) maxMemory) / MEGABYTE;


        MemoryStat memoryStat = new MemoryStat();

        memoryStat.usedMemoryString = memoryNumberFormat.format(usedMemoryInMegs) + "M";
        memoryStat.maxMemoryString = memoryNumberFormat.format(maxMemoryInMegs) + "M";
        memoryStat.totalMemoryString = memoryNumberFormat.format(totalMemoryInMegs) + "M";
        memoryStat.percentMemoryUsedString = percentMemoryFreeNumberFormat.format(percentMemoryUsed) + "%";
        return memoryStat;
    }
    
    public static String getPercentMemoryFreeString(){
        MemoryStat memoryStat = getMemoryStat();
        return "[memory used: " + memoryStat.usedMemoryString + " / " + memoryStat.maxMemoryString  + " (" + memoryStat.totalMemoryString  + ") = " + memoryStat.percentMemoryUsedString + "]";
    }
}
