package edu.jhmi.telometer.view.about;

import edu.jhmi.telometer.interfce.HardConstants;
import edu.jhmi.telometer.util.FrameUtil;
import edu.jhmi.telometer.util.api.ImageDirUtil;
import lombok.extern.slf4j.Slf4j;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;


@Slf4j
public final class AboutDialog {
    private final JButton closeButton = new JButton("Close");
    private final JButton licenseButton = new JButton("License");
    private final JButton environmentButton = new JButton("Environment");
    private final JPanel mainPanel;
    private final EnvironmentDialog environmentDialog = new EnvironmentDialog();
    private final JLabel mainLabel = new JLabel(all());
    private JDialog dialog;

    private ImageDirUtil imageDirUtil;


    private String all() {

        String all = "<HTML>"
                + "<BR/><BR/>"
                + "Version: <STRONG> " + HardConstants.VERSION + "</STRONG> <BR/>"
                + "Storage Directory: <STRONG> " + getStorageDirectoryString() + "</STRONG> <BR/>"
                + "More Info: <U> http://demarzolab.pathology.jhmi.edu/telometer</U> <BR/>"
                + "<BR/><BR/>"
                + "<div align=center> Credits </div> <BR/>"
                + "Programming: Jim Morgan, Joe Zimmerman<BR/>"
                + "Concept: Alan Meeker, Ph.D.<BR/>"
                + "<BR/><BR/>"
                + "<div align=center> Contact </div> <BR/>"
                + "For questions or comments about Telometer, please contact Dr. Alan Meeker at:<BR/>"
                + "<U>ameeker1@jhmi.edu</U> <BR/>"
                + "The developer, Jim Morgan, can be reached at: <U>jmorgan@jhmi.edu</U> <BR/>"
                + "<BR/><BR/>"
                + "<div align=center> Backups </div> <BR/>"
                + "<STRONG> We no longer store your data remotely.</STRONG><BR/>"
                + "<STRONG> YOU MUST DO YOUR OWN BACKUPS, </STRONG><BR/>"
                + "<STRONG> OTHERWISE YOUR DATA COULD BE LOST. </STRONG><BR/>"
                + "See the 'Storage Directory' above to see where your data is stored.<BR/>"
                + "<BR/><BR/>"
                + "<div align=center> About </div> <BR/>"
                + "Telometer  -- An ImageJ plugin for quantifying telomere (or other FISH) signals <BR>"
                + "Copyright (C) 2022 The Johns Hopkins University<BR>"
                + "<BR/>"
                + "This program is free software: you can redistribute it and/or modify<BR/>"
                + "it under the terms of the GNU General Public License as published by <BR/>"
                + "the Free Software Foundation, either version 3 of the License, or <BR/>"
                + "(at your option) any later version. <BR/>"
                + "<BR/>"
                + "This program is distributed in the hope that it will be useful,<BR/"
                + "but <STRONG>WITHOUT ANY WARRANTY</STRONG>; without even the implied warranty of <BR/"
                + "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the <BR/"
                + "GNU General Public License for more details. <BR/>"
                + "<BR/><BR/>"
                + "</HTML>";
        return all;
    }

    private String getStorageDirectoryString() {
        if (imageDirUtil == null){
            return "null";
        }
        File dir = imageDirUtil.getDataDir();
        try{
            return dir.getCanonicalPath();
        } catch (IOException e){
            log.error("could not get canonical path: " + e);
            return dir.getAbsolutePath();
        }
    }




    public AboutDialog() {

        JPanel centerPanel = new JPanel(new BorderLayout());
        centerPanel.add(mainLabel);

        ActionListener actionListener = new ActionListener(){
        	public void actionPerformed(ActionEvent e){
        		Object source = e.getSource();
        		if (source == closeButton){
                    dialog.dispose();
        		} else if (source == licenseButton){
                    GplDialog.displayGPL(dialog);
        		} else if (source == environmentButton){
                    environmentDialog.displayDialog(dialog);
                } else {
        			throw new RuntimeException("Bad ActionEvent: " + e);
        		}
        	}
        };
        closeButton.addActionListener(actionListener);
        licenseButton.addActionListener(actionListener);
        environmentButton.addActionListener(actionListener);

        JPanel buttonPanel = new JPanel();
        buttonPanel.add(closeButton);
        buttonPanel.add(licenseButton);
        buttonPanel.add(environmentButton);

        mainPanel = new JPanel(new BorderLayout());
        mainPanel.add(centerPanel, BorderLayout.CENTER);
        mainPanel.add(buttonPanel, BorderLayout.SOUTH);
    }

    public void showDialog(Frame parentFrame) {
        log.info("Displaying AboutDialog");
        mainLabel.setText(all());
        dialog = new JDialog(parentFrame, "About Telometer", true);
        dialog.getContentPane().add(mainPanel);
        dialog.pack();
        FrameUtil.setRelativeLocation(dialog, parentFrame);
        dialog.setVisible(true);
    }

    public void setImageDirUtil(ImageDirUtil imageDirUtil) {
        this.imageDirUtil = imageDirUtil;
    }
}
