package edu.jhmi.telometer.view.admin;

import edu.jhmi.telometer.bean.AdminType;
import edu.jhmi.telometer.bean.Preset;
import edu.jhmi.telometer.interfce.Dialogable;
import edu.jhmi.telometer.interfce.Named;
import edu.jhmi.telometer.interfce.OkListener;
import edu.jhmi.telometer.service.api.AdminService;
import edu.jhmi.telometer.thirdparty.SpringUtilities;
import edu.jhmi.telometer.util.ClassUtil;
import edu.jhmi.telometer.util.ComboBoxUtil;
import edu.jhmi.telometer.util.DialogUtil;
import edu.jhmi.telometer.util.DoubleUtil;
import edu.jhmi.telometer.util.FrameUtil;
import edu.jhmi.telometer.util.IconUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.Assert;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SpringLayout;
import javax.swing.border.Border;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

@Slf4j
public class AdminPanel {

    private final JFrame frame = new JFrame("Admin");

    private JComboBox<AdminType> adminTypeComboBox = new JComboBox<>(AdminType.values());
    private JComboBox<Named> namedComboBox = new JComboBox();


    private final JPanel mainPanel = new JPanel(new BorderLayout());
    private final JButton editButton = new JButton("Edit", IconUtil.getIcon("edit"));
    private final JButton deleteButton = new JButton("Delete", IconUtil.getIcon("remove"));
    private final JButton createButton = new JButton("Create", IconUtil.getIcon("add"));
    private final JButton deleteScoringButton = new JButton("Delete Scoring", IconUtil.getIcon("removeAlt"));
    private AdminService adminService;

    public AdminPanel() {
        setToolTips();
        FrameUtil.setInitialLocation(frame);
        JPanel choicesPanel = new JPanel(new SpringLayout());
        JLabel[] labels = {
                new JLabel("Table:"),
                new JLabel("Choices:")
        };
        JComponent[] components = {
                adminTypeComboBox,
                namedComboBox
        };
        for (int i = 0; i < labels.length; i++){
            choicesPanel.add(labels[i]);
            choicesPanel.add(components[i]);
            labels[i].setToolTipText(components[i].getToolTipText());
        }

        SpringUtilities.makeCompactGrid(choicesPanel, 2, 2, 5, 5, 5, 5);


        JPanel actionPanel = new JPanel(new SpringLayout());
        actionPanel.add(editButton);
        actionPanel.add(deleteButton);
        actionPanel.add(createButton);
        SpringUtilities.makeCompactGrid(actionPanel, 1, 3, 5, 5, 5, 5);

        JPanel choicesAndActionsPanel = new JPanel(new BorderLayout());
        choicesAndActionsPanel.add(choicesPanel, BorderLayout.NORTH);
        choicesAndActionsPanel.add(actionPanel, BorderLayout.SOUTH);

        JPanel panel1 = new JPanel();
        panel1.add(deleteScoringButton);
        JPanel bottomPanel = new JPanel(new BorderLayout());
        bottomPanel.add(panel1, BorderLayout.WEST);
        mainPanel.add(choicesAndActionsPanel, BorderLayout.NORTH);
        mainPanel.add(bottomPanel, BorderLayout.SOUTH);

        deleteButton.addActionListener(l -> deleteButtonClicked());
        editButton.addActionListener(l -> editButtonClicked());
        createButton.addActionListener(l -> createButtonClicked());
        adminTypeComboBox.addActionListener(l -> adminTypeComboBoxChanged());
        deleteScoringButton.addActionListener(l -> deleteScoringButtonClicked());

        frame.add(getMainPanel());
        frame.pack();
        int screenWidth = (int) Toolkit.getDefaultToolkit().getScreenSize().getWidth();
        int width = (int) Math.max(screenWidth / 2, frame.getPreferredSize().getWidth() + 50);
        int height = (int) frame.getPreferredSize().getHeight() + 50;
        frame.setSize(new Dimension(width, height));
    }

    private void setToolTips() {
        adminTypeComboBox.setToolTipText("<HTML>Select the table you want to edit<BR/>Choices will automatically appear below</HTML>");
        namedComboBox.setToolTipText("The choices in the above table");
        createButton.setToolTipText("Add another choice to the table");
        editButton.setToolTipText("Edit the selected choice");
        deleteButton.setToolTipText("Delete the selected choice from the table");
        deleteScoringButton.setToolTipText("Delete a Scoring (and its images)");
    }


    private <T extends Named> void setChoices(List<T> choices) {
        namedComboBox.setModel(ComboBoxUtil.listToComboBoxModel(choices));
    }

    private void showMessage(String message) {
        DialogUtil.displayInfoDialog(mainPanel, message);
    }

    private <T> void setSelectedChoice(T choice) {
        namedComboBox.setSelectedItem(choice);
    }

    private Dialogable dialogable;

    private void disposeDialogable() {
        if (dialogable != null) {
            dialogable.dispose();
        }
    }

    private void deleteScoringButtonClicked() {
        dialogable = new ScoringIdInputDialog(deleteScoringButton);
        dialogable.setOkListener(new OkListener<Integer>() {
            @Override
            public void okayButtonClicked(Integer scoringId) {
                log.info("ScoringId selected: {}", scoringId);
                try {
                    if (!adminService.scoringIdExists(scoringId)) {
                        showMessage(String.format("No scoring with Id#%s", scoringId));
                        return;
                    }
                    adminService.deleteScoring(scoringId);
                    showMessage(String.format("Scoring#%s deleted", scoringId));
                    disposeDialogable();
                } catch (RuntimeException e) {
                    log.error("Could not delete scoring: " + e, e);
                    showMessage(String.format("Could not delete scoring. Error:%s", e.getMessage()));
                }
            }
        });
        dialogable.display();
    }

    private void adminTypeComboBoxChanged() {
        AdminType adminType = (AdminType) adminTypeComboBox.getSelectedItem();
        refreshList(adminType.getClazz());
    }

    private void createButtonClicked() {
        disposeDialogable();
        AdminType adminType = (AdminType) adminTypeComboBox.getSelectedItem();
        switch (adminType) {
            case PRESET: {
                dialogable = new PresetDialog(editButton,
                        Preset.builder().id(null).name("myPreset").minSpotSize(10).maxSpotSize(350).rollingBallSize(5)
                                .despeckleRatio1000(DoubleUtil.doubleToLong1000(0.5)).build(), false);
                break;
            }
            case PROJECT:
            case CELL_TYPE:
            case TISSUE_TYPE:
            case LESION_TYPE: {
                Named named = (Named) ClassUtil.newInstance(adminType.getClazz());
                String defaultName = String.format("my%s", named.getClass().getSimpleName());
                named.setName(defaultName);
                dialogable = new NamedDialog(editButton, copyObject(named));
                break;
            }
            default:
                throw new RuntimeException("bad adminType: " + adminType);
        }
        dialogable.setOkListener(new OkListener<Named>() {
            @Override
            public void okayButtonClicked(Named named) {
                if (adminService.idWithName(named.getClass(), named.getName()) != null) {
                    showMessage(String.format("Name already exists: '%s'", named.getName()));
                    return;
                }
                if (named.getName().trim().equals("")) {
                    showMessage("new name is empty");
                    return;
                }
                alterNamed(named);
            }

        });
        dialogable.display();
    }


    private void editButtonClicked() {
        Named named = (Named) namedComboBox.getSelectedItem();
        if (named == null) {
            showMessage("No item chosen to edit.\nUse 'Create' button to create an item if necessary.");
            return;
        }
        disposeDialogable();
        AdminType adminType = AdminType.clazzToLookupType(named.getClass());
        switch (adminType) {
            case PRESET: {
                boolean isReferenced = adminService.isReferenced(named);
                dialogable = new PresetDialog(editButton, (Preset) copyObject(named), isReferenced);
                break;
            }
            case PROJECT:
            case CELL_TYPE:
            case TISSUE_TYPE:
            case LESION_TYPE: {
                dialogable = new NamedDialog(editButton, copyObject(named));
                break;
            }
            default:
                throw new RuntimeException("bad adminType: " + adminType);
        }

        dialogable.setOkListener(new OkListener<Named>() {
            @Override
            public void okayButtonClicked(Named named) {
                String newName = named.getName();
                Long existingId = adminService.idWithName(named.getClass(), newName);
                if (existingId != null && !existingId.equals(named.getId())) {
                    showMessage(String.format("Name already exists: '%s'", newName));
                    return;
                }
                if (newName.trim().equals("")) {
                    showMessage("new name is empty");
                    return;
                }
                alterNamed(named);
            }
        });
        dialogable.display();

    }

    private Named copyObject(Named named) {
        Named copy = (Named) ClassUtil.newInstance(named.getClass());
        if (named instanceof Preset) {
            Preset source = (Preset) named;
            return Preset.builder()
                    .id(source.getId())
                    .name(source.getName())
                    .maxSpotSize(source.getMaxSpotSize())
                    .minSpotSize(source.getMinSpotSize())
                    .despeckleRatio1000(source.getDespeckleRatio1000())
                    .rollingBallSize(source.getRollingBallSize())
                    .build();
        }
        copy.setId(named.getId());
        copy.setName(named.getName());
        return copy;
    }

    private void deleteButtonClicked() {
        Named named = (Named) namedComboBox.getSelectedItem();
        if (named == null) {
            showMessage("No item selected to delete.\nUse 'Create' button to create an item if necessary.");
            return;
        }
        if (adminService.isReferenced(named)){
            showMessage("Can't delete because it's referenced by other items: " + named);
            return;
        }
        adminService.deleteItem(named);
        refreshList(named.getClass());
    }

    private JPanel getMainPanel() {
        return mainPanel;
    }

    private <T extends Named> void refreshList(Class<T> clazz) {
        List<T> choices = adminService.findAllItems(clazz);
        setChoices(choices);
    }

    private void alterNamed(Named named) {
        adminService.alterItem(named);
        disposeDialogable();
        refreshList(named.getClass());
        setSelectedChoice(named);
    }


    public void setAdminService(AdminService adminService) {
        this.adminService = adminService;
    }

    public void display(){
        log.info("Displaying AdminPanel");
        adminTypeComboBoxChanged();
        frame.setVisible(true);
    }
}
