package edu.jhmi.telometer.view.admin;

import edu.jhmi.telometer.interfce.Dialogable;
import edu.jhmi.telometer.interfce.Named;
import edu.jhmi.telometer.interfce.OkListener;
import edu.jhmi.telometer.thirdparty.SpringUtilities;
import edu.jhmi.telometer.util.DialogUtil;
import edu.jhmi.telometer.util.FrameUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.Assert;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SpringLayout;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

@Slf4j
public class NamedDialog<T extends Named> implements Dialogable {

    private OkListener<T> okListener;


    private final JTextField idTextField = new JTextField();
    private final JTextField nameTextField = new JTextField("myName");
    private final JPanel mainPanel = new JPanel(new BorderLayout());
    private final JButton okButton = new JButton("Ok");
    private final JButton cancelButton = new JButton("Cancel");
    private final JDialog dialog;
    private final T named;

    public NamedDialog(Component component, T named){
        Assert.notNull(named, "named is null");
        this.named = named;
        idTextField.setEditable(false);
        idTextField.setBackground(Color.LIGHT_GRAY);
        mainPanel.add(getNorthPanel(named), BorderLayout.NORTH);
        mainPanel.add(getSouthPanel(), BorderLayout.SOUTH);
        mainPanel.add(getFormPanel(), BorderLayout.CENTER);

        idTextField.setText(named.getId() != null ? named.getId() + "" : "");
        nameTextField.setText(named.getName());

        okButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                okayButtonClicked();
            }
        });

        Frame frame = JOptionPane.getFrameForComponent(component);
        dialog = new JDialog(frame, named.getClass().getSimpleName(), true);
        cancelButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                dialog.dispose();
            }
        });
        dialog.getContentPane().add(mainPanel);
        dialog.pack();
        FrameUtil.setRelativeLocation(dialog, component);
    }



    private void okayButtonClicked() {
        String id = idTextField.getText();
        String name = nameTextField.getText();

        named.setName(name);
        String namedErrorMessage = getNamedErrorMessage(named);
        if (namedErrorMessage != null){
            showMessage(namedErrorMessage);
            return;
        }
        okListener.okayButtonClicked(named);
    }

    private String getNamedErrorMessage(Named named) {
        if (named.getName().trim().equals("")){
            return "name must have a value";
        }
        return null;
    }


    private JPanel getNorthPanel(T named) {

        String labelName = String.format("%s %s", named.getId() != null ? "Edit" : "Create",
                named.getClass().getSimpleName());
        JPanel panel = new JPanel();
        panel.add(new JLabel(labelName));
        return panel;
    }

    private JPanel getSouthPanel(){
        JPanel panel = new JPanel();
        panel.add(okButton);
        panel.add(cancelButton);
        return panel;
    }


    private JPanel getFormPanel() {
        JPanel formPanel = new JPanel(new SpringLayout());

        JLabel[] labels = {
                new JLabel("ID"),
                new JLabel("Name"),
        };

        JComponent[] components = {
                idTextField,
                nameTextField,
        };
        for (int i = 0; i < labels.length; i++) {
            formPanel.add(labels[i]);
            formPanel.add(components[i]);
        }
        SpringUtilities.makeCompactGrid(formPanel,
                labels.length, 2,
                2, 20,
                20, 15);
        return formPanel;
    }

    private JPanel getMainPanel() {
        return mainPanel;
    }

    public void setOkListener(OkListener okListener) {
        this.okListener = okListener;
    }

    private void showMessage(String message){
        DialogUtil.displayInfoDialog(mainPanel, message);
    }


    public void display() {
        dialog.setVisible(true);

    }


    public void dispose() {
        dialog.setVisible(false);
        dialog.dispose();
    }
}
