package edu.jhmi.telometer.view.query.table;

import edu.jhmi.telometer.bean.ColumnSpec;
import edu.jhmi.telometer.interfce.Translator;
import org.springframework.util.Assert;

import javax.swing.table.AbstractTableModel;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class GeneralTableModel<T> extends AbstractTableModel {
    private final List<T> rows = new ArrayList<>();
    private final List<ColumnSpec<T>> columnSpecs;

    public GeneralTableModel(List<ColumnSpec<T>> columnSpecs) {
        this.columnSpecs = new ArrayList(columnSpecs);
    }

    private void checkColumnNumber(int column) {
        Assert.isTrue(column >= 0 && column < getColumnCount(), "bad column number: " + column);
    }

    private void checkRowNumber(int row) {
        Assert.isTrue(row >= 0 && row < getRowCount(), "bad row: " + row + " rowNumbers: " + getRowCount());
    }


    public String getColumnName(int column) {
        checkColumnNumber(column);
        return columnSpecs.get(column).getColumnName();
    }

    @Override
    public int getRowCount() {
        return rows.size();
    }

    @Override
    public int getColumnCount() {
        Assert.notNull(columnSpecs, "columnSpecs is null");
        return columnSpecs.size();
    }

    @Override
    public Object getValueAt(int row, int column) {
        checkRowNumber(row);
        checkColumnNumber(column);
        T t = rows.get(row);
        Translator<T> translator = columnSpecs.get(column).getTranslator();
        Object object = translator.objectToString(t);
        return object;
    }


    public void setRows(List<T> newRows) {
        this.rows.clear();
        this.rows.addAll(newRows);
        fireTableDataChanged();
    }

    public List<T> getRows() {
        return new ArrayList<T>(rows);
    }
}
