package edu.jhmi.telometer.view.score;

import edu.jhmi.telometer.bean.CellType;
import edu.jhmi.telometer.bean.LesionType;
import edu.jhmi.telometer.bean.NucleusEntry;
import edu.jhmi.telometer.bean.TissueType;
import edu.jhmi.telometer.interfce.ScorePanelListener;
import edu.jhmi.telometer.util.ComboBoxUtil;
import edu.jhmi.telometer.util.IconUtil;
import edu.jhmi.telometer.util.IntDocumentFilter;
import edu.jhmi.telometer.thirdparty.SpringUtilities;
import lombok.extern.slf4j.Slf4j;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SpringLayout;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.PlainDocument;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


@Slf4j
public class NucleiiPanel {


    private final JComboBox tissueTypeComboBox = new JComboBox(ComboBoxUtil.DEFAULT_ITEMS);
    private final JComboBox lesionTypeComboBox = new JComboBox(ComboBoxUtil.DEFAULT_ITEMS);
    private final JComboBox cellTypeComboBox = new JComboBox(ComboBoxUtil.DEFAULT_ITEMS);
    private final JTextField nameTextField = new JTextField();
    private final JTextField cellCountTextField = new JTextField();
    private final JTextField notesTextField = new JTextField();
    private final JButton addButton = new JButton("Add Nucleus", IconUtil.getIcon("add"));
    private final JButton removeButton = new JButton("Remove Nucleus", IconUtil.getIcon("remove"));
    private ScorePanelListener scorePanelListener;
    private final DefaultListModel<NucleusEntry> existingNucleusListModel = new DefaultListModel();
    private final JList nucleusJlist = new JList(existingNucleusListModel);
    private final JPanel mainPanel = new JPanel(new BorderLayout());


    private boolean isInitted = false;

    public NucleiiPanel() {
        initGuiIfNecessary();
    }


    private void initGuiIfNecessary() {
        if (isInitted) {
            return;
        }
        PlainDocument doc = (PlainDocument) cellCountTextField.getDocument();
        doc.setDocumentFilter(new IntDocumentFilter()); //only allow int in cellCountTextField

        mainPanel.add(BorderLayout.WEST, initNewPanel());
        mainPanel.add(BorderLayout.EAST, initExistingPanel());
        //displayFrame(mainPanel);

        addButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                fireAddNucleusButtonClicked();
            }
        });

        removeButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                fireRemoveNucleusButtonClicked();
            }
        });
        addButton.setToolTipText("<HTML>Add the nucleus and ROI (region of interest) <BR/>"
                + "Note: When outlining the nucleii, draw the outline closely to the DAPI stained nuclear <BR/>"
                + "outline, avoiding dark pixels outside of the nucleus as much as possible.</HTML>");
        removeButton.setToolTipText("remove the nucleus and ROI (region of interest)");
    }

    private void fireRemoveNucleusButtonClicked() {
        NucleusEntry nucleusEntry = getExistingNucleusEntry();
        scorePanelListener.removeNucleusButtonClicked(nucleusEntry);
        if (existingNucleusListModel.getSize() > 0) {
            nucleusJlist.setSelectedIndex(existingNucleusListModel.getSize() - 1);
        }
    }

    public void fireAddNucleusButtonClicked() {
        NucleusEntry nucleusEntry = getNewNucleusEntry();
        scorePanelListener.addNucleusButtonClicked(nucleusEntry);
    }

    private NucleusEntry getExistingNucleusEntry() {
        int selectedIndex = nucleusJlist.getSelectedIndex();
        if (selectedIndex == -1) {
            return null;
        }
        NucleusEntry nucleusEntry = existingNucleusListModel.getElementAt(selectedIndex);
        return nucleusEntry;
    }

    private NucleusEntry getNewNucleusEntry() {
        NucleusEntry nucleusEntry = NucleusEntry.builder()
                .name(nameTextField.getText())
                .cellType((CellType) cellTypeComboBox.getSelectedItem())
                .lesionType((LesionType) lesionTypeComboBox.getSelectedItem())
                .tissueType((TissueType) tissueTypeComboBox.getSelectedItem())
                .notes(notesTextField.getText())
                .cellCount(getCellCount())
                .build();
        return nucleusEntry;
    }

    private void setNucleusEntry(NucleusEntry nucleusEntry) {
        nameTextField.setText(nucleusEntry.getName());
        cellCountTextField.setText(nucleusEntry.getCellCount() != null ? nucleusEntry.getCellCount() + "" : "");
        notesTextField.setText(nucleusEntry.getNotes());
        setComboBoxItem(lesionTypeComboBox, nucleusEntry.getLesionType());
        setComboBoxItem(tissueTypeComboBox, nucleusEntry.getTissueType());
        setComboBoxItem(cellTypeComboBox, nucleusEntry.getCellType());

    }

    private static void setComboBoxItem(JComboBox comboBox, Object object) {
        if (object == null) {
            return;
        }
        comboBox.setSelectedItem(object);
    }

    private Integer getCellCount() {
        String cellCount = cellCountTextField.getText();
        Integer cellCountInt = cellCount.matches("\\d+") ? Integer.parseInt(cellCount) : null;
        return cellCountInt;
    }


    private JPanel initNewPanel() {
        JPanel formPanel = new JPanel();

        JLabel[] labels = {
                new JLabel("Name"),
                new JLabel("Tissue Type"),
                new JLabel("Lesion Type"),
                new JLabel("Cell Type"),
                new JLabel("Cell Count"),
                new JLabel("Notes"),
        };

        JComponent[] components = {
                nameTextField,
                tissueTypeComboBox,
                lesionTypeComboBox,
                cellTypeComboBox,
                cellCountTextField,
                notesTextField
        };
        for (int i = 0; i < labels.length; i++) {
            formPanel.add(labels[i]);
            formPanel.add(components[i]);
        }
        formPanel.setLayout(new SpringLayout());
        SpringUtilities.makeCompactGrid(formPanel,
                labels.length, 2,
                6, 6,
                6, 6);

        JPanel southPanel = new JPanel();
        southPanel.add(addButton);

        JPanel panel = new JPanel(new BorderLayout());
        panel.add(formPanel, BorderLayout.CENTER);
        panel.add(southPanel, BorderLayout.SOUTH);

        return panel;
    }

    private JPanel initExistingPanel() {
        removeButton.setEnabled(false);
        JPanel northPanel = new JPanel(new BorderLayout());
        northPanel.add(new JLabel("Nucleii:"));
        nucleusJlist.setVisibleRowCount(10);
        nucleusJlist.addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent listSelectionEvent) {
                int selectedIndex = nucleusJlist.getSelectedIndex();
                removeButton.setEnabled(selectedIndex != -1);
            }
        });
        JScrollPane scrollPane = new JScrollPane(nucleusJlist);

        JPanel southPanel = new JPanel();
        southPanel.add(removeButton);

        JPanel panel = new JPanel(new BorderLayout());
        panel.add(northPanel, BorderLayout.NORTH);
        panel.add(scrollPane, BorderLayout.CENTER);
        panel.add(southPanel, BorderLayout.SOUTH);
        return panel;
    }

    public void setScorePanelListener(ScorePanelListener scorePanelListener) {
        this.scorePanelListener = scorePanelListener;
    }

    public void addNucleusEntry(NucleusEntry nucleusEntry) {
        existingNucleusListModel.addElement(nucleusEntry);
    }

    public void removeNucleusEntryByName(String name) {
        int index = getIndexOfName(name);
        if (index == -1) {
            return;
        }
        existingNucleusListModel.removeElementAt(index);
    }

    private int getIndexOfName(String name) {
        for (int i = 0; i < existingNucleusListModel.getSize(); i++) {
            if (existingNucleusListModel.getElementAt(i).getName().equals(name)) {
                return i;
            }
        }
        return -1;
    }

    public void setTissueTypes(List<TissueType> tissueTypes) {
        tissueTypeComboBox.setModel(ComboBoxUtil.listToComboBoxModel(tissueTypes));
    }

    public void setCellTypes(List<CellType> cellTypes) {
        cellTypeComboBox.setModel(ComboBoxUtil.listToComboBoxModel(cellTypes));
    }

    public void setLesionTypes(List<LesionType> lesionTypes) {
        lesionTypeComboBox.setModel(ComboBoxUtil.listToComboBoxModel(lesionTypes));
    }


    public JPanel getPanel() {
        return mainPanel;
    }

    public void clear() {
        existingNucleusListModel.clear();
    }

    public void setName(String name) {
        nameTextField.setText(name);
    }
}
