package edu.jhmi.telometer.view.score;

import edu.jhmi.telometer.bean.ToolMode;
import edu.jhmi.telometer.interfce.ScorePanelListener;
import edu.jhmi.telometer.util.IconUtil;

import javax.swing.JButton;
import javax.swing.JPanel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

public class ToolbarPanel {

    private final JPanel mainPanel = new JPanel();

    private final JButton drawOnButton = new JButton("Draw", IconUtil.getIcon("edit"));
    private final JButton handButton = new JButton("Hand", IconUtil.getIcon("hand"));
    private final JButton zoomButton = new JButton("Zoom", IconUtil.getIcon("zoom"));
    private final JButton[] buttons = {drawOnButton, handButton, zoomButton};

    private ScorePanelListener scorePanelListener;

    private boolean isInitted;

    public ToolbarPanel() {
        initGuiIfNecessary();
    }

    private void initGuiIfNecessary() {
        if (isInitted) {
            return;
        }

        Arrays.stream(buttons).forEach(b -> mainPanel.add(b));

        drawOnButton.addActionListener(a -> scorePanelListener.fireDrawButtonClicked());
        handButton.addActionListener(a -> scorePanelListener.fireHandButtonClicked());
        zoomButton.addActionListener(a -> scorePanelListener.fireZoomButtonClicked());

        isInitted = true;
        handButton.setToolTipText("activate Hand Tool on the Image (used to move around on the image)");
        zoomButton.setToolTipText("activate Zoom Tool on the Image (or use + or - to zoom)");
        drawOnButton.setToolTipText("activate Draw Tool on the Image (used to draw on the image)");
    }

    private ToolMode toolModeForButton(JButton button) {
        if (button == zoomButton) {
            return ToolMode.ZOOM;
        } else if (button == drawOnButton) {
            return ToolMode.DRAW;
        } else if (button == handButton) {
            return ToolMode.HAND;
        } else {
            throw new RuntimeException("bad button source: " + button);
        }
    }

    public void setToolMode(ToolMode toolMode) {
        for (JButton button : buttons) {
            boolean isDisabled = toolMode == toolModeForButton(button);
            button.setEnabled(!isDisabled);
        }
    }

    public JPanel getPanel() {
        return mainPanel;
    }

    public void setScorePanelListener(ScorePanelListener scorePanelListener) {
        this.scorePanelListener = scorePanelListener;
    }

}