# Telometer Paper

## Introduction

  Telomeres shorten in cancer and pre-cancerous lesions, therefore, the
measurement of telomere lengths in situ will facilitate studies on
cancer pathogenesis. To overcome the limitations posed by traditional
Southern-based methods, we recently developed a technique for telomere
length analysis in fixed tissue samples. In this method, quantitative
data can be extracted using existing digital image analysis software.
However, several of the image processing steps are cumbersome,
requiring extensive manual intervention. We therefore have developed a
custom software plugin for extracting telomere length data from
fluorescence images using the analysis program ImageJ. This software
streamlines the process of quantitative telomere length analysis and
allows for direct export of results to an established MS Access
database.

## Design

  Quantification of the digitized fluorescent signals is accomplished
through the image analysis software package ImageJ
(http://rsb.info.nih.gov/ij/) and the custom designed plugin as
follows: matched telomeric and nuclear DNA grayscale image files are
captured using a Zeiss Axioskop epifluorescence microscope equipped
with appropriate fluorescence filter sets (Carl Zeiss Inc, Thornwood,
NY; Omega Optical, Brattleboro, VT) and a cooled CCD camera (Micro MAX
digital camera, Princeton Instruments, Trenton, NJ). The images are
first normalized with a simple background subtraction, and the
resulting telomere image is run through a sharpening filter, followed
by enhancement using a rolling ball algorithm (http://rsb.info.nih.gov/ij/developer/api/ij/plugin/filter/BackgroundSubtracter.html.)
for contouring of telomeric spots, followed by manual thresholding to
remove any remaining background noise. Binarized masks are then created
and applied to the original fluorescence images. For a given cell,
telomeric signals identified by the segment mask, which are contained
within the area inscribed by the nuclear DNA signal, are then measured,
and the data for each telomeric spot tabulated. The total DAPI (DNA)
fluorescence signal for each nucleus is likewise quantified. For each
cell, the individual telomere intensities are summed, and this total is
divided by the total DAPI fluorescence signal for that nucleus. This
normalization to the nuclear DAPI signal corrects for differences in
nuclear cutting planes and ploidy. Tabulated data is then stored in an 
H2 database on the user's computer.

## Results

  The newly written ImageJ plugin described above performed equivalently
to the prior, more labor-intensive method, when assessed on standard
curve validation image sets. In particular, use of the rolling ball
algorithm provides rapid and superior delineation of telomeric signals
in the context of high levels of background autofluorescence, as is
commonly the case with formalin-fixed tissue specimens. Overall, the
new plugin largely eliminates the need for manual intervention and
should be applicable to other situations requiring quantification of
fluorescence signals, such as with FISH images.

## Conclusion

  The newly developed plugin for ImageJ accelerates the process of
extracting quantitative data from fluorescence microscopy images. The
new program features superior discrimination of fluorescent signals and
substantially reduces the need for user intervention during image
processing. This software has proven useful in telomere length analysis
and should be broadly applicable to other FISH studies where
quantification is desired.